@@ -1,6 +1,6 @@
 package Net::SFTP::Foreign::Compat;
 
-our $VERSION = '1.70_05';
+our $VERSION = '0.90_17';
 
 use warnings;
 use strict;
@@ -37,32 +37,6 @@ sub import {
     }
 }
 
-our %DEFAULTS = ( put => [best_effort => 1],
-                  get => [best_effort => 1],
-                  ls  => [],
-                  new => [] );
-
-BEGIN {
-    my @forbidden = qw( setcwd cwd open opendir sftpread sftpwrite
-                        seek tell eof write flush read getc lstat stat
-                        fstat remove rmdir mkdir setstat fsetstat
-                        close closedir readdir realpath readlink
-                        rename symlink abort get_content join glob
-                        rremove rget rput error die_on_error );
-
-    for my $method (@forbidden) {
-        my $super = "SUPER::$method";
-        no strict 'refs';
-        *{$method} = sub {
-            unless (index((caller)[0], "Net::SFTP::Foreign") == 0) {
-                croak "Method '$method' is not available from " . __PACKAGE__
-                    . ", use the real Net::SFTP::Foreign if you want it!";
-            }
-            shift->$super(@_);
-        };
-    }
-}
-
 sub new {
     my ($class, $host, %opts) = @_;
 
@@ -71,10 +45,10 @@ sub new {
 	$warn = delete($opts{warn}) || sub {};
     }
     else {
-	$warn = sub { warn(CORE::join '', @_, "\n") };
+	$warn = sub { warn(join '', @_, "\n") };
     }
 
-    my $sftp = $class->SUPER::new($host, @{$DEFAULTS{new}}, %opts);
+    my $sftp = $class->SUPER::new($host, %opts);
 
     $sftp->{_compat_warn} = $warn;
 
@@ -91,7 +65,7 @@ sub _warn {
 
 sub _warn_error {
     my $sftp = shift;
-    if (my $e = $sftp->SUPER::error) {
+    if (my $e = $sftp->error) {
 	$sftp->_warn($e);
     }
 }
@@ -102,55 +76,45 @@ sub status {
 }
 
 sub get {
-    croak '$Usage: $sftp->get($local, $remote, $cb)' if @_ < 2 or @_ > 4;
     my ($sftp, $remote, $local, $cb) = @_;
 
     my $save = defined(wantarray);
     my @content;
-    my @cb;
-    if (defined $cb or $save) {
-        @cb = ( callback => sub {
-                    my ($sftp, $data, $off, $size) = @_;
-                    $cb->($sftp, $data, $off, $size) if $cb;
-                    push @content, $data if $save
-                });
-    }
 
     $sftp->SUPER::get($remote, $local,
-                      @{$DEFAULTS{get}},
-                      dont_save => !defined($local),
-                      @cb)
-        or return undef;
+		      dont_save => !defined($local),
+		      callback => sub {
+			  my ($sftp, $data, $off, $size) = @_;
+			  $cb->($sftp, $data, $off, $size) if $cb;
+			  push @content, $data if $save
+		      } )
+	or return undef;
 
     if ($save) {
-	return CORE::join('', @content);
+	return join('', @content);
     }
 }
 
 sub put {
-    croak '$Usage: $sftp->put($local, $remote, $cb)' if @_ < 3 or @_ > 4;
     my ($sftp, $local, $remote, $cb) = @_;
 
     $sftp->SUPER::put($local, $remote,
-                      @{$DEFAULTS{put}},
-		      callback => $cb);
+		      (defined $cb ? (callback => $cb) : ()));
     $sftp->_warn_error;
-    !$sftp->SUPER::error;
+    !$sftp->error;
 }
 
 sub ls {
-    croak '$Usage: $sftp->ls($path, $cb)' if @_ < 2 or @_ > 3;
     my ($sftp, $path, $cb) = @_;
     if ($cb) {
 	$sftp->SUPER::ls($path,
-                         @{$DEFAULTS{ls}},
 			 wanted => sub { _rebless_attrs($_[1]->{a});
 					 $cb->($_[1]);
 					 0 } );
 	return ();
     }
     else {
-	if (my $ls = $sftp->SUPER::ls($path, @{$DEFAULTS{ls}})) {
+	if (my $ls = $sftp->SUPER::ls($path)) {
 	    _rebless_attrs($_->{a}) for @$ls;
 	    return @$ls;
 	}
@@ -158,15 +122,15 @@ sub ls {
     }
 }
 
-sub do_open { shift->SUPER::open(@_) }
+sub do_open { shift->open(@_) }
 
-sub do_opendir { shift->SUPER::opendir(@_) }
+sub do_opendir { shift->opendir(@_) }
 
-sub do_realpath { shift->SUPER::realpath(@_) }
+sub do_realpath { shift->realpath(@_) }
 
 sub do_read {
     my $sftp = shift;
-    my $read = $sftp->SUPER::sftpread(@_);
+    my $read = $sftp->sftpread(@_);
     $sftp->_warn_error;
     if (wantarray) {
 	return ($read, $sftp->status);
@@ -177,7 +141,7 @@ sub do_read {
 }
 
 sub _gen_do_and_status {
-    my $method = "SUPER::" . shift;
+    my $method = shift;
     return sub {
 	my $sftp = shift;
 	$sftp->$method(@_);
@@ -189,7 +153,7 @@ sub _gen_do_and_status {
 *do_write = _gen_do_and_status('sftpwrite');
 *do_close = _gen_do_and_status('close');
 *do_setstat = _gen_do_and_status('setstat');
-*do_fsetstat = _gen_do_and_status('setstat');
+*do_fsetstat = _gen_do_and_status('fsetstat');
 *do_remove = _gen_do_and_status('remove');
 *do_rename = _gen_do_and_status('rename');
 *do_mkdir = _gen_do_and_status('mkdir');
@@ -206,10 +170,8 @@ sub _rebless_attrs {
 }
 
 sub _gen_do_stat {
-    my $name = shift;
-    my $method = "SUPER::$name";
+    my $method = shift;
     return sub {
-        croak '$Usage: $sftp->'.$name.'($local, $remote, $cb)' if @_ != 2;
 	my $sftp = shift;
 	if (my $a = $sftp->$method(@_)) {
 	    return _rebless_attrs($a);
@@ -258,37 +220,9 @@ the C<Net::SFTP> and L<Net::SFTP::Attributes> packages so no other
 parts of the program have to modified in order to move from Net::SFTP
 to Net::SFTP::Foreign.
 
-=head2 Setting defaults
-
-The hash C<%Net::SFTP::Foreign::DEFAULTS> can be used to set default
-values for L<Net::SFTP::Foreign> methods called under the hood and
-otherwise not accesible through the Net::SFTP API.
-
-The entries currently supported are:
-
-=over
-
-=item new => \@opts
-
-extra options passed to Net::SFTP::Foreign constructor.
-
-=item get => \@opts
-
-extra options passed to Net::SFTP::Foreign::get method.
-
-=item put => \@opts
-
-extra options passed to Net::SFTP::Foreign::put method.
-
-=item ls  => \@opts
-
-extra options passed to Net::SFTP::Foreign::ls method.
-
-=back
-
 =head1 COPYRIGHT
 
-Copyright (c) 2006-2008, 2011 Salvador FandiE<ntilde>o
+Copyright (c) 2006 Salvador FandiE<ntilde>o
 
 All rights reserved.  This program is free software; you can
 redistribute it and/or modify it under the same terms as Perl itself.
